/*
 * $Id: DNSEvent.java 292 2006-02-28 23:30:47Z davidb $
 *
 * Copyright (c) 2005 VeriSign, Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

package se.rfc.unbound;

import java.util.*;

/**
 * This is the core event class. A DNSEvent represents either a request or a
 * request, response pair. Note that a request may be modified by the
 * resolution process, so this class keeps track of the original request.
 * 
 * DNSEvents are frequently created in response to the needs of another event.
 * The new event is chained to the old event forming a dependency chain.
 * 
 * @author davidb
 * @version $Revision: 292 $
 */
public class DNSEvent implements Cloneable
{

  /**
   * This is the current, mutable request -- this request will change based on
   * the current needs of a module.
   */
  private Request  mCurrentRequest;
  /**
   * This is the original, immutable request. This request must not be changed
   * after being set.
   */
  private Request  mOriginalRequest;

  /**
   * This is the normal response to the current request. It may be modified as
   * it travels through the chain, but only the most recent is relevant.
   */
  private Response mResponse;

  /**
   * If event was created on behalf of another event, the "next" event is that
   * original event. That is, if this is not null, some other event is
   * (possibly) waiting on the completion of this one.
   */
  private DNSEvent mForEvent;

  /**
   * This is a map of per-module state. Each resolver module has the
   * opportunity to create a custom per-event state object that it can attach
   * to this event. The keys for this map are references to the module itself,
   * typically.
   */
  private Map      mModuleStateMap;


  /**
   * This is the dependency depth of this event -- in other words, the length
   * of the "nextEvent" chain.
   */
  private int      mDepth;

  /** If this event is closed, then it must not be used. */
  private boolean  mIsClosed;

  /**
   * Create a empty event. This is typically only used internally.
   */
  protected DNSEvent()
  {
    mModuleStateMap = new HashMap();
  }

  /**
   * Create a request event.
   * 
   * @param request The initial request.
   */
  public DNSEvent(Request request)
  {
    this();
    mOriginalRequest = request;
    mCurrentRequest = (Request) request.clone();
  }

  /**
   * Create a local, dependent event.
   * 
   * @param request The initial request.
   * @param forEvent The dependent event.
   */
  public DNSEvent(Request request, DNSEvent forEvent)
  {
    this(request);

    mForEvent = forEvent;
    mDepth = forEvent.getDepth() + 1;
  }

  /**
   * @return The current request.
   */
  public Request getRequest()
  {
    return mCurrentRequest;
  }

  /**
   * @return The original request. Do not modify this!
   */
  public Request getOrigRequest()
  {
    return mOriginalRequest;
  }

  /**
   * @return The "for" event. I.e., the event that is depending on this event.
   */
  public DNSEvent forEvent()
  {
    return mForEvent;
  }

  /**
   * @return The response that has been attached to this event, or null if one
   *         hasn't been attached yet.
   */
  public Response getResponse()
  {
    return mResponse;
  }

  /**
   * Attach a response to this event.
   * 
   * @param response The response to attach. The must match the current
   *          request at time of attachment.
   */
  public void setResponse(Response response)
  {
    mResponse = response;
  }

  /**
   * Attach a response to this event.
   * 
   * @param response The response message to attach. The must match the
   *          current request at time of attachment.
   */
  public void setResponse(SMessage response)
  {
    mResponse = new Response(response, Response.OK);
  }
  
  /**
   * Close the event. This doesn't actually destroy the event, it just marks
   * it as closed.
   */
  public void close()
  {
    mIsClosed = true;
  }

  /** Resets the event back to the request state. */
  public void reset()
  {
    mResponse = null;
  }

  /**
   * Fetch any attached per-module state for this event.
   * 
   * @param module A reference for the module itself. This is the key.
   * @return A state object for the module, or null if one wasn't attached.
   */
  public Object getModuleState(Resolver module)
  {
    return mModuleStateMap.get(module);
  }

  /**
   * Attach per-module state to this event.
   * 
   * @param module A reference for the module itself. This is the key.
   * @param state A state object.
   */
  public void setModuleState(Resolver module, Object state)
  {
    mModuleStateMap.put(module, state);
  }

  /**
   * @return The depth of this event. The depth is the events position in a
   *         dependency chain of events.
   */
  public int getDepth()
  {
    return mDepth;
  }

  /**
   * @return true if this event has been closed.
   */
  public boolean isClosed()
  {
    return mIsClosed;
  }

  /**
   * Clone the event.
   */
  public Object clone()
  {
    try
    {
      DNSEvent event = (DNSEvent) super.clone();
      
      return event;
    }
    catch (CloneNotSupportedException e)
    {
      return null;
    }
  }

  /**
   * @return A string representation of the event, to be used in logging,
   *         perhaps.
   */
  public String toString()
  {
    return super.toString() + " " + mCurrentRequest.toString();
  }
}
